--[[
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program. If not, see http://www.gnu.org/licenses/.

Attempting to generate top-down 2D terrain using noise and cellular automata.

]]--

local helptext = [[Welcome to the noisy cellular automata map generator demo!
It generates a lake with outcroppings and islands. It also has depth in the water.
Press "up"/"down" arrows to adjust the iterations of cellular automata, press "left"/"right" arrows to change the noise seed, and press "ins"/"del" to increase/decrease the noise density.]]

local cellmap = require("cellmap")
local lake = nil

function love.load()

  lake = cellmap:generate(80, 80, 0, 0.3, 6)

end

function love.update(dt)

end

function love.draw()

  -- print help
  love.graphics.setColor(255, 255, 255)
  love.graphics.printf(helptext, 0, 0, 600)
  love.graphics.print(string.format("seed: %d\ndensity: %f\niter: %s", lake.seed, lake.density, lake.iterations), 650, 60)

  -- draw the map bigger
  love.graphics.translate(0, 60)
  love.graphics.scale(6, 6)

  for x=1, lake.width do
    for y=1, lake.height do

      local ground = lake.contour[x][y]

      if ground then
        love.graphics.setColor(92, 64, 32)
        love.graphics.rectangle("fill", x, y, 1, 1)
      else
        -- draw lake depth
        local shallow = lake.depth[x][y]
        if shallow then
          love.graphics.setColor(32, 64, 32)
        else
          love.graphics.setColor(16, 16, 64)
        end
        love.graphics.rectangle("fill", x, y, 1, 1)
      end

    end
  end

end

function love.keypressed(key)
  if key == "escape" then
    love.event.quit()
  elseif key == "left" then
    lake = cellmap:generate(
      lake.width, lake.height, math.max(0, lake.seed - 1), lake.density, lake.iterations)
  elseif key == "right" then
    lake = cellmap:generate(
      lake.width, lake.height, math.max(0, lake.seed + 1), lake.density, lake.iterations)
  elseif key == "up" then
    lake = cellmap:generate(
      lake.width, lake.height, lake.seed, lake.density, math.max(0, lake.iterations + 1))
  elseif key == "down" then
    lake = cellmap:generate(
      lake.width, lake.height, lake.seed, lake.density, math.max(0, lake.iterations - 1))
  elseif key == "insert" then
    lake = cellmap:generate(
      lake.width, lake.height, lake.seed, math.max(0, lake.density + .025), lake.iterations)
  elseif key == "delete" then
    lake = cellmap:generate(
      lake.width, lake.height, lake.seed, math.max(0, lake.density - .025), lake.iterations)
  end
end
